#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatbook
# @Time         : 2023/9/19 10:28
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

# 根据文章长短选用4k 16k 32k

from meutils.pipe import *

from chatllm.llmchain.applications import ChatBase


class ChatBook(ChatBase):
    pass
